@@ -29,6 +29,7 @@
  
 using System;
 using Gtk;
+using Mono.Unix;
 
 namespace Banshee
 {
@@ -70,6 +71,18 @@ namespace Banshee
 		}
 	}
 	
+    public class MessageDialogs
+    {
+        public static void CannotRenamePlaylist()
+        {
+            HigMessageDialog.RunHigMessageDialog(null,
+            DialogFlags.Modal, MessageType.Error, ButtonsType.Ok,
+            Catalog.GetString("Cannot Rename Playlist"),
+            Catalog.GetString("A playlist with this name already exists. " + 
+            "Please choose another name."));
+        }
+    }
+	
 	public class ErrorDialog
 	{
 		static public void Run(string message)
@@ -155,6 +168,10 @@ namespace Banshee
 			dialog.AddActionWidget(cancelButton, ResponseType.Cancel);
 			dialog.AddActionWidget(saveButton, ResponseType.Ok);
 
+			saveButton.CanDefault = true;
+			saveButton.HasDefault = true;
+			entry.ActivatesDefault = true;
+
 			ResponseType result = (ResponseType)dialog.Run();
 			string input = entry.Text;
 			
