@@ -42,13 +42,16 @@ namespace Banshee
 		public Database Db;
 		public LibraryTransactionManager TransactionManager;
 		public Hashtable Tracks;
+		public Hashtable TracksFnKeyed;
 		public Hashtable Playlists;
 		
 		public event EventHandler Reloaded;
+		public event EventHandler Updated;
 		
 		public Library()
 		{
 			Tracks = new Hashtable();
+			TracksFnKeyed = new Hashtable();
 			Playlists = new Hashtable();
 			ReloadDatabase();
 			TransactionManager = new LibraryTransactionManager();
@@ -112,6 +115,28 @@ namespace Banshee
 				return libraryLocation;		 	
 			 }	
 		}
+		
+		public void SetTrack(int id, TrackInfo track)
+		{
+		    Tracks[id] = track;
+		    TracksFnKeyed[MakeFilenameKey(track.Uri)] = track;
+		    
+		    EventHandler handler = Updated;
+		    if(handler != null)
+		      handler(this, new EventArgs());
+		}
+		
+		public static string MakeFilenameKey(string uri)
+		{
+		      string path = uri;
+		      
+		      if(path.StartsWith("file://"))
+		          path = path.Substring(7);
+		
+    		      return Path.GetDirectoryName(path) + 
+                     Path.DirectorySeparatorChar + 
+                     Path.GetFileNameWithoutExtension(path);
+		}
 	}
 
 	public enum SourceType : uint {
@@ -126,6 +151,7 @@ namespace Banshee
 		protected string name;
 		protected SourceType type;
 		protected bool canEject;
+		protected bool canRename;
 
 		public event EventHandler Updated;
 		
@@ -140,13 +166,18 @@ namespace Banshee
 			get {
 				return name;
 			}
-			
-			set {
-				UpdateName(name, value);
-				EventHandler handler = Updated;
-				if(handler != null)
-					handler(this, new EventArgs());
-			}
+		}
+		
+		public bool Rename(string newName)
+		{
+		    if(!UpdateName(name, newName))
+				    return false;
+				    
+        		EventHandler handler = Updated;
+        		if(handler != null)
+        			handler(this, new EventArgs());
+        			
+        	   return true;
 		}
 		
 		public SourceType Type
@@ -168,9 +199,16 @@ namespace Banshee
 			}
 		}
 		
-		public virtual void UpdateName(string oldName, string newName)
+		public bool CanRename
 		{
+		    get {
+		        return canRename;
+		    }
+		}
 		
+		public virtual bool UpdateName(string oldName, string newName)
+		{
+		  return false;
 		}
 		
 		public virtual bool Eject()
@@ -181,9 +219,10 @@ namespace Banshee
 	
 	public class LibrarySource : Source
 	{
-		public LibrarySource() : base(Catalog.GetString("Library"), SourceType.Library)
+		public LibrarySource() : base(Catalog.GetString("Library"), 
+		  SourceType.Library)
 		{
-		
+		      canRename = false;
 		}
 		
 		public override int Count
@@ -200,17 +239,25 @@ namespace Banshee
 	
 		public PlaylistSource(string name) : base(name, SourceType.Playlist)
 		{
-		
+		    canRename = true;
 		}
 		
-		public override void UpdateName(string oldName, string newName)
+		public override bool UpdateName(string oldName, string newName)
 		{
 			if(oldName.Equals(newName))
-				return;
+				return false;
 			
 			Playlist pl = new Playlist(oldName);
-			pl.Rename(newName);
-			name = newName;
+			if(pl.Rename(newName)) {
+			     name = newName;
+			     return true;
+			}
+			
+			Core.ThreadEnter();
+			MessageDialogs.CannotRenamePlaylist();
+			Core.ThreadLeave();
+			
+			return false;
 		}
 		
 		public override int Count
@@ -235,15 +282,18 @@ namespace Banshee
 		{
 			this.device = device;
 			canEject = true;
+			canRename = true;
 		}
 		
-		public override void UpdateName(string oldName, string newName)
+		public override bool UpdateName(string oldName, string newName)
 		{
 			if(!oldName.Equals(newName)) {
 				device.Name = newName;
 				name = newName;
 				device.Save();
 			}
+			
+			return true;
 		}
 		
 		public override int Count
@@ -316,6 +366,7 @@ namespace Banshee
 			this.disk = disk;
 			disk.Updated += OnUpdated;
 			canEject = true;
+			canRename = false;
 		}
 		
 		public override int Count
@@ -341,7 +392,7 @@ namespace Banshee
 		private void OnUpdated(object o, EventArgs args)
 		{
 			Core.ThreadEnter();
-			Name = disk.Title;
+			name = disk.Title;
 			Core.ThreadLeave();
 		}
 	}
@@ -527,14 +578,20 @@ namespace Banshee
 				Append(ti);
 		}
 		
-		public void Rename(string newName)
+		public bool Rename(string newName)
 		{
+			if(GetId(newName) != 0)
+			   return false;
+			
 			Statement query = new Update("Playlists", "Name", newName) + 
 				new Where("PlaylistID", Op.EqualTo, Playlist.GetId(name));
 			try {
 				Core.Library.Db.Execute(query);
 				name = newName;
-			} catch(Exception) {}
+				return true;
+			} catch(Exception) {
+			     return false;
+			}
 		}
 		
 		public void Save()
