@@ -65,6 +65,22 @@ namespace Banshee
 			canSaveToDatabase = true;
 		}
 	
+        private void CheckIfExists(string uri)
+        {
+              bool exists = false;
+              try {
+                exists = Core.Library.TracksFnKeyed[Library.MakeFilenameKey(uri)] != null;
+              } catch(Exception) {
+                exists = false;
+              }
+              
+              if(exists) {
+                  // TODO: we should actually probably take this as a hint to
+                  // reparse metadata
+                  throw new ApplicationException("Song is already in library");
+              } 
+        }
+	
 	    public LibraryTrackInfo(string uri, string artist, string album, 
 	       string title, string genre, uint trackNumber, uint trackCount,
 	       int year, long duration)
@@ -83,11 +99,12 @@ namespace Banshee
 			this.year = year;
 			this.duration = duration;
 			
+			CheckIfExists(uri);
+			
 			SaveToDatabase(true);
+			Core.Library.SetTrack(trackId, this);
 			
-			Core.Library.Tracks[trackId] = this;
-			
-			uid = Core.Instance.NextUid;
+			uid = UidGenerator.Next;
 			PreviousTrack = Gtk.TreeIter.Zero;
 	    }
 	    
@@ -104,21 +121,23 @@ namespace Banshee
 				uri = uri.Substring(6);
 				LoadFromDatabase(uri);
 			} else {
-				if(!LoadFromDatabase(uri))
+                CheckIfExists(uri);
+                
+            		if(!LoadFromDatabase(uri))
 					LoadFromFile(uri);
 			}
 			
-			Core.Library.Tracks[trackId] = this;
+			Core.Library.SetTrack(trackId, this);
 			
-			uid = Core.Instance.NextUid;
+			uid = UidGenerator.Next;
 			PreviousTrack = Gtk.TreeIter.Zero;
 		}
 		
 		public LibraryTrackInfo(IDataReader reader) : this()
 		{
 			LoadFromDatabaseReader(reader);
-			Core.Library.Tracks[trackId] = this;
-			uid = Core.Instance.NextUid;
+			Core.Library.SetTrack(trackId, this);;
+			uid = UidGenerator.Next;
 			PreviousTrack = Gtk.TreeIter.Zero;
 		}
 		
