@@ -130,10 +130,7 @@ namespace Banshee
 			
 			Core.Instance.DBusServer.RegisterObject(
 				new BansheeCore(Window, this), "/org/gnome/Banshee/Core");
-			
-			Core.Instance.AudioCdCore.Updated += OnAudioCdCoreUpdated;
-			AudioCdRefresh();
-				
+						
 			WindowPlayer.Show();
 			
 			Core.Instance.Player.Iterate += OnPlayerTick;
@@ -142,6 +139,9 @@ namespace Banshee
 			Core.Instance.AudioCdPlayer.Iterate += OnPlayerTick;
 			Core.Instance.AudioCdPlayer.EndOfStream += OnPlayerEos;	
 			
+			Core.Instance.AudioCdCore.DiskRemoved += OnAudioCdCoreDiskRemoved;
+			Core.Instance.AudioCdCore.Updated += OnAudioCdCoreUpdated;
+			
 			LoadSettings();
 			Core.Instance.PlayerInterface = this;
 			
@@ -531,7 +531,8 @@ namespace Banshee
       
      	private void SetInfoLabel(string text)
       	{
-      		LabelInfo.Markup = "<span size=\"small\">" + text + "</span>";
+      		LabelInfo.Markup = "<span size=\"small\">" 
+      		    + GLib.Markup.EscapeText(text) + "</span>";
       	}
       	
       	public void TogglePlaying()
@@ -586,21 +587,6 @@ namespace Banshee
             ti.IncrementPlayCount();
             playlistView.QueueDraw();
         }
-
-		private void AudioCdRefresh()
-		{
-			Core.ThreadEnter();
-			
-			sourceView.QueueDraw();
-			playlistView.QueueDraw();
-			
-			Core.ThreadLeave();
-		}
-
-		private void OnAudioCdCoreUpdated(object o, EventArgs args)
-		{
-			AudioCdRefresh();
-		}
 		
 		// ---- Window Event Handlers ----
 		
@@ -745,7 +731,8 @@ namespace Banshee
 				// Translators: position in song. eg, "0:37 of 3:48"
 				String.Format(Catalog.GetString("{0} of {1}"),
 					      String.Format("{0}:{1:00}", position / 60, position % 60),
-					      String.Format("{0}:{1:00}", activeTrackInfo.Duration / 60, activeTrackInfo.Duration % 60))
+					      String.Format("{0}:{1:00}", activeTrackInfo.Duration / 60, 
+					      activeTrackInfo.Duration % 60))
 			);	
 		}
 		
@@ -824,16 +811,27 @@ namespace Banshee
 			FileChooserDialog chooser = new FileChooserDialog(
 				Catalog.GetString("Import Folder to Library"),
 				null,
-				FileChooserAction.SelectFolder,
-				"gnome-vfs"
+				FileChooserAction.SelectFolder
 			);
 			
+			
+			try {
+			     chooser.SetCurrentFolderUri(Core.GconfClient.Get(
+			         GConfKeys.LastFileSelectorUri) as string);
+			} catch(Exception) {
+			     chooser.SetCurrentFolder(Environment.GetFolderPath(
+				    Environment.SpecialFolder.Personal));
+		    }
+		    
 			chooser.AddButton(Stock.Cancel, ResponseType.Cancel);
 			chooser.AddButton(Stock.Open, ResponseType.Ok);
 			chooser.DefaultResponse = ResponseType.Ok;
 			
 			if(chooser.Run() == (int)ResponseType.Ok) 
 				playlistModel.AddFile(chooser.CurrentFolderUri);
+				
+			Core.GconfClient.Set(GConfKeys.LastFileSelectorUri,
+			     chooser.CurrentFolderUri);
 			
 			chooser.Destroy();
 		}
@@ -853,10 +851,17 @@ namespace Banshee
 			FileChooserDialog chooser = new FileChooserDialog(
 				Catalog.GetString("Import Files to Library"),
 				null,
-				FileChooserAction.Open,
-				"gnome-vfs"
+				FileChooserAction.Open
 			);
 			
+			try {
+			     chooser.SetCurrentFolderUri(Core.GconfClient.Get(
+			         GConfKeys.LastFileSelectorUri) as string);
+			} catch(Exception) {
+			     chooser.SetCurrentFolder(Environment.GetFolderPath(
+				    Environment.SpecialFolder.Personal));
+		    }
+			
 			chooser.AddButton(Stock.Cancel, ResponseType.Cancel);
 			chooser.AddButton(Stock.Open, ResponseType.Ok);
 			
@@ -868,6 +873,9 @@ namespace Banshee
 					playlistModel.AddFile(path);
 			}
 			
+			Core.GconfClient.Set(GConfKeys.LastFileSelectorUri,
+			     chooser.CurrentFolderUri);
+			
 			chooser.Destroy();
 		}
 				
@@ -918,7 +926,7 @@ namespace Banshee
 				Core.ThreadEnter();
 				(gxml["ViewNameLabel"] as Label).Markup = 
 					String.Format(Catalog.GetString("<b>{0}'s Music Library</b>"),
-						Core.Instance.UserFirstName);
+						GLib.Markup.EscapeText(Core.Instance.UserFirstName));
 				
 				Core.ThreadLeave();
 			} else if(source.Type == SourceType.Ipod) {
@@ -933,11 +941,6 @@ namespace Banshee
 				string tooltip = ipodSource.DiskUsageString + " (" +
 					ipodSource.DiskAvailableString + ")";
 				toolTips.SetTip(ipodDiskUsageBar, tooltip, tooltip);
-				
-				Core.ThreadEnter();
-				(gxml["ViewNameLabel"] as Label).Markup = 
-					"<b>" + source.Name + "</b>";
-				Core.ThreadLeave();
 			} else if(source.Type == SourceType.AudioCd) {
 				playlistModel.Clear();
 				playlistModel.Source = source;
@@ -947,17 +950,60 @@ namespace Banshee
 			} else {
 				playlistModel.LoadFromPlaylist(source.Name);
 				playlistModel.Source = source;
-				
-				Core.ThreadEnter();
-				(gxml["ViewNameLabel"] as Label).Markup = 
-					"<b>" + source.Name + "</b>";
-				Core.ThreadLeave();
 			}
 			
+            Core.ThreadEnter();
+            (gxml["ViewNameLabel"] as Label).Markup = 
+                "<b>" + GLib.Markup.EscapeText(source.Name) + "</b>";
+            Core.ThreadLeave();
+			
 			gxml["ButtonRip"].Visible = source.Type == SourceType.AudioCd;
 			gxml["IpodContainer"].Visible = source.Type == SourceType.Ipod;
+			gxml["SearchLabel"].Sensitive = source.Type == SourceType.Ipod 
+			     || source.Type == SourceType.Library;
+			searchEntry.Sensitive = gxml["SearchLabel"].Sensitive;
 		}
 		
+        private void OnAudioCdCoreDiskRemoved(object o, 
+            AudioCdCoreDiskRemovedArgs args)
+        {
+            Source source = sourceView.SelectedSource;
+            if(source == null)
+                return;
+
+            if(source.Type == SourceType.AudioCd) {
+                AudioCdSource cdSource = source as AudioCdSource;
+                if(cdSource.Disk.Udi == args.Udi) {
+                    sourceView.SelectLibrary();
+                }
+            }
+        }
+
+        private void OnAudioCdCoreUpdated(object o, EventArgs args)
+        {
+            Source source = sourceView.SelectedSource;
+            if(source == null)
+                return;
+            
+            Core.ThreadEnter();
+            playlistView.QueueDraw();
+            
+            if(source.Type == SourceType.AudioCd 
+                && playlistModel.FirstTrack != null 
+                && playlistModel.FirstTrack.GetType() == 
+                typeof(AudioCdTrackInfo)) {
+                AudioCdSource cdSource = source as AudioCdSource;
+                AudioCdTrackInfo track = playlistModel.FirstTrack 
+                    as AudioCdTrackInfo;
+                
+                if(cdSource.Disk.Udi == track.Udi) 
+                    (gxml["ViewNameLabel"] as Label).Markup = 
+                    "<b>" + GLib.Markup.EscapeText(cdSource.Disk.Title) + "</b>";
+            }
+                    
+            Core.ThreadLeave();
+        }
+		
 		private void OnIpodPropertiesClicked(object o, EventArgs args)
 		{
 			if(sourceView.SelectedSource.Type != SourceType.Ipod)
@@ -1059,7 +1105,7 @@ namespace Banshee
 					Catalog.GetString("{0} Total Play Time"),
 					timeDisp);
 				text += " ";
-				text += String.Format("[{0}]", playlistModel.TotalDuration);
+				//text += String.Format("[{0}]", playlistModel.TotalDuration);
 				LabelStatusBar.Text = text;
 			}
 				
@@ -1074,6 +1120,11 @@ namespace Banshee
 			playlistView.ColumnChooser();
 		}
 		
+		private void OnMenuColumnsActivate(object o, EventArgs args)
+		{
+		    OnItemColumnsActivate(o, args);
+		}
+		
 		private void OnItemRemoveActivate(object o, EventArgs args)
 		{
 			int selCount = playlistView.Selection.CountSelectedRows();
@@ -1179,19 +1230,17 @@ namespace Banshee
 			Menu menu = gxmlSourceMenu["SourceMenu"] as Menu;
 			MenuItem addSelectedSongs = gxmlSourceMenu["ItemAddSelectedSongs"] as MenuItem;
 			MenuItem sourceDuplicate = gxmlSourceMenu["ItemSourceDuplicate"] as MenuItem;
+			MenuItem sourceRename = gxmlSourceMenu["ItemSourceRename"] as MenuItem;			
 			MenuItem sourceDelete = gxmlSourceMenu["ItemSourceDelete"] as MenuItem;
 			MenuItem sourceProperties = gxmlSourceMenu["ItemSourceProperties"] as MenuItem;
 			ImageMenuItem ejectItem = gxmlSourceMenu["ItemEject"] as ImageMenuItem;
 			
-			addSelectedSongs.Sensitive = source.Type == SourceType.Playlist
-				&& playlistView.Selection.CountSelectedRows() > 0;
-			sourceDuplicate.Sensitive = false;
-			
-			if(source.Type == SourceType.Ipod)
-				sourceProperties.Sensitive = true;
-			else
-				sourceProperties.Sensitive = false;
-		
+			//addSelectedSongs.Sensitive = source.Type == SourceType.Playlist
+			//	&& playlistView.Selection.CountSelectedRows() > 0;
+            addSelectedSongs.Visible = false;
+            
+		    sourceProperties.Visible = source.Type == SourceType.Ipod;
+		  
 			if(source.CanEject) {
 				ejectItem.Image = new Gtk.Image("media-eject", IconSize.Menu);
 			}
@@ -1199,9 +1248,13 @@ namespace Banshee
 			menu.Popup(null, null, null, IntPtr.Zero, 0, args.Event.Time);
 			menu.Show();
 			
-			sourceDuplicate.Visible = !source.CanEject;
+			addSelectedSongs.Visible = source.Type == SourceType.Playlist ||
+			 source.Type == SourceType.Ipod;
+			
+			sourceDuplicate.Visible = false;
 			ejectItem.Visible = source.CanEject;
 			sourceDelete.Visible = !source.CanEject;
+			sourceRename.Visible = source.CanRename;
 			
 			args.RetVal = true;
 		}
@@ -1245,18 +1298,28 @@ namespace Banshee
 		
 		private void OnItemSourceRenameActivate(object o, EventArgs args)
 		{
-			if(sourceView.HighlightedSource == null || 
-				sourceView.HighlightedSource.Type == SourceType.Library)
-				return;
-				
-			InputDialog input = new InputDialog(
+			Source source = sourceView.HighlightedSource;
+			
+			if(source == null || !source.CanRename)
+			     return;
+			
+			InputDialog input;
+			
+			if(source.Type == SourceType.Playlist)
+			 input = new InputDialog(
 				Catalog.GetString("Rename Playlist"),
 				Catalog.GetString("Enter new playlist name"), 
-				"playlist-icon-large.png", 
-				sourceView.HighlightedSource.Name);
+				"playlist-icon-large.png", source.Name);
+		    else
+		      input = new InputDialog(
+		          Catalog.GetString("Rename iPod"),
+		          Catalog.GetString("Enter new name for your iPod"),
+		          "ipod-48.png", source.Name);
+				
 			string newName = input.Execute();
 			if(newName != null)
-				sourceView.HighlightedSource.Name = newName;
+				source.Rename(newName);
+
 			sourceView.QueueDraw();
 		}
 		
@@ -1379,6 +1442,7 @@ namespace Banshee
 		
 			if(path == null)
 				return;
+			
 			clickX = (int)args.Event.X;
 			clickY = (int)args.Event.Y;
 		
@@ -1412,6 +1476,7 @@ namespace Banshee
 				return;
 			if(args.Event.Window != playlistView.BinWindow)
 				return;
+			        
 			args.RetVal = true;
 			if(!Gtk.Drag.CheckThreshold(playlistView, clickX, clickY,
 						    (int)args.Event.X, (int)args.Event.Y))
@@ -1421,6 +1486,9 @@ namespace Banshee
 						       (int)args.Event.Y, out path))
 				return;
 
+           if(sourceView.SelectedSource.Type == SourceType.AudioCd)
+		      return;
+		      
 			Gtk.Drag.Begin(playlistView, new TargetList (playlistViewSourceEntries),
 				       Gdk.DragAction.Move | Gdk.DragAction.Copy, 1, args.Event);
 		}
@@ -1698,6 +1766,7 @@ namespace Banshee
 				
 			switch(source.Type) {
 				case SourceType.Library:
+				case SourceType.AudioCd:
 					return;
 			}
 				
@@ -1822,7 +1891,7 @@ namespace Banshee
 					propWin.Destroy();
 					if(propWin.Edited && device.CanWrite)
 						device.Save();
-					source.Name = device.Name;
+					source.Rename(device.Name);
 					sourceView.QueueDraw();
 					break;
 			}
